@@ -112,7 +112,7 @@ IDocument::getErrorMessage (DocumentError errorCode)
             errorMessage = g_strdup (_("File I/O error."));
             break;
         default:
-            errorMessage = g_strdup_printf (_("Unknown error (%d)."), errorCode);            
+            errorMessage = g_strdup_printf (_("Unknown error (%d)."), errorCode);
     }
 
     g_assert (NULL != errorMessage && "The error message is NULL");
@@ -185,7 +185,7 @@ void
 IDocument::attach (const IDocumentObserver *observer)
 {
     g_assert (NULL != observer && "Tried to attach a NULL observer.");
-    
+
     m_Observers = g_list_prepend (m_Observers, (gpointer)observer);
 }
 
@@ -224,13 +224,13 @@ IDocument::notifyFindChanged (gint pageNum, DocumentRectangle *matchRect)
         m_FindRect = new DocumentRectangle (*matchRect);
     }
     m_FindPage = pageNum;
-                
+
     for ( GList *item = g_list_first (m_Observers) ; NULL != item ;
           item = g_list_next (item) )
     {
         IDocumentObserver *observer = (IDocumentObserver *)item->data;
         observer->notifyFindChanged (matchRect);
-    } 
+    }
 }
 
 ///
@@ -276,9 +276,12 @@ IDocument::notifyFindStarted ()
 void
 IDocument::notifyLoad ()
 {
-    // Add the two first pages to the cache.
+    // Add the two first pages, if they exists, to the cache.
     addPageToCache (1);
-    addPageToCache (2);
+    if ( 1 < getNumPages () )
+    {
+        addPageToCache (2);
+    }
 
     for ( GList *item = g_list_first (m_Observers) ; NULL != item ;
           item = g_list_next (item) )
@@ -435,6 +438,42 @@ IDocument::notifyReload ()
     }
 }
 
+///
+/// @brief The document has been saved.
+///
+/// This is called when the JobSave class is done. It in turn notifies
+/// all attached observers about this situation.
+///
+void
+IDocument::notifySave ()
+{
+    for ( GList *item = g_list_first (m_Observers) ; NULL != item ;
+          item = g_list_next (item) )
+    {
+        IDocumentObserver *observer = (IDocumentObserver *)item->data;
+        observer->notifySave ();
+    }
+}
+
+///
+/// @brief The document couldn't be saved.
+///
+/// This is called by the JobSave class when the document couldn't
+/// be saved. It in turns notifies all attached observers.
+///
+/// @param error The error message of why couldn't save the document.
+///
+void
+IDocument::notifySaveError (const GError *error)
+{
+    for ( GList *item = g_list_first (m_Observers) ; NULL != item ;
+          item = g_list_next (item) )
+    {
+        IDocumentObserver *observer = (IDocumentObserver *)item->data;
+        observer->notifySaveError (error);
+    }
+}
+
 ///
 /// @brief Loads a file.
 ///
@@ -450,7 +489,7 @@ IDocument::load (const gchar *fileName, const gchar *password)
     m_CurrentPage = 1;
     m_Rotation = 0;
     m_Scale = 1.0f;
-    
+
     JobLoad *job = new JobLoad ();
     job->setDocument (this);
     job->setFileName (fileName);
@@ -474,6 +513,22 @@ IDocument::reload (void)
     IJob::enqueue (job);
 }
 
+///
+/// @brief Saves a copy of the document.
+///
+/// Saves the current document in a new file.
+///
+/// @param fileName The file name to save the document's copy to.
+///
+void
+IDocument::save (const gchar *fileName)
+{
+    JobSave *job = new JobSave ();
+    job->setDocument (this);
+    job->setFileName (fileName);
+    IJob::enqueue (job);
+}
+
 ///
 /// @brief Gets the document's title.
 ///
@@ -847,7 +902,7 @@ IDocument::getNumPages ()
 ///         if no password was used.
 ///
 const gchar *
-IDocument::getPassword ()
+IDocument::getPassword () const
 {
     return m_Password;
 }
@@ -883,7 +938,7 @@ IDocument::setNumPages (gint numPages)
 ///         if the document isn't loaded yet.
 ///
 const gchar *
-IDocument::getFileName ()
+IDocument::getFileName () const
 {
     if ( NULL == m_FileName )
     {
@@ -1066,7 +1121,7 @@ IDocument::goToPage (gint pageNum)
     if ( pageNum != m_CurrentPage && 1 <= pageNum && pageNum <= getNumPages ())
     {
         m_CurrentPage = pageNum;
-        
+
         addPageToCache (m_CurrentPage);
         if ( 1 < m_CurrentPage )
         {
@@ -1192,10 +1247,10 @@ IDocument::getZoom (void)
 void
 IDocument::setZoom (gdouble zoom)
 {
-    if ( ABS (zoom - m_Scale) > 0.00001 )    
+    if ( ABS (zoom - m_Scale) > 0.00001 )
     {
         G_LOCK (JobRender);
-        m_Scale = zoom;
+        m_Scale = CLAMP (zoom, ZOOM_OUT_MAX, ZOOM_IN_MAX);
         refreshCache ();
         G_UNLOCK (JobRender);
         notifyPageZoomed ();
@@ -1475,10 +1530,10 @@ IDocument::activateLinkAtPosition (gint x, gint y)
     // XXX For now only non rotated pages.
     if ( NULL != page && 0 == getRotation () )
     {
-        DocumentLink *link = page->getLinkAtPosition (x, y);
+        IDocumentLink *link = page->getLinkAtPosition (x, y);
         if ( NULL != link )
         {
-            goToPage (link->getDestinationPage ());
+            link->activate (this);
         }
-    }    
+    }
 }
