@@ -65,7 +65,7 @@ namespace ePDFView
             ///               otherwise.
             ///
             virtual void activeZoomFit (gboolean active) = 0;
-           
+
             ///
             /// @brief Actives or deactivates the zoom to width.
             ///
@@ -76,7 +76,7 @@ namespace ePDFView
             ///               otherwise.
             ///
             virtual void activeZoomWidth (gboolean active) = 0;
-           
+
             ///
             /// @brief Shows the open file dialog.
             ///
@@ -105,7 +105,23 @@ namespace ePDFView
             ///         This string will be freed by the presenter.
             ///
             virtual gchar *promptPasswordDialog (void) = 0;
-            
+
+            ///
+            /// @brief Shows the save file dialog.
+            ///
+            /// The view must ask the user which file name to use to
+            /// save using the toolkit's specific save dialog.
+            ///
+            /// @param lastFolder The last folder used to save a file. This is
+            ///                   used to show this folder when the save dialog
+            ///                   appears.
+            ///
+            /// @return A copy of the file name that the user will try to use
+            ///         to save or NULL if the user cancelled the operation.
+            ///         This string will be freed by the presenter.
+            ///
+            virtual gchar *saveFileDialog (const gchar *lastFolder) = 0;
+
             ///
             /// @brief Changes the sensitivity of the "Find" action.
             ///
@@ -131,7 +147,7 @@ namespace ePDFView
             ///                  insensitive (disable) it.
             ///
             virtual void sensitiveGoToFirstPage (gboolean sensitive) = 0;
-            
+
             ///
             /// @brief Changes the sensitivity of the "Go To Last Page" action.
             ///
@@ -144,7 +160,7 @@ namespace ePDFView
             ///                  insensitive (disable) it.
             ///
             virtual void sensitiveGoToLastPage (gboolean sensitive) = 0;
-            
+
             ///
             /// @brief Changes the sensitivity of the "Go To Next Page" action.
             ///
@@ -157,7 +173,7 @@ namespace ePDFView
             ///                  insensitive (disable) it.
             ///
             virtual void sensitiveGoToNextPage (gboolean sensitive) = 0;
-            
+
             ///
             /// @brief Changes the sensitivity of the "Go To Page" action.
             ///
@@ -170,7 +186,7 @@ namespace ePDFView
             ///                  insensitive (disable) it.
             ///
             virtual void sensitiveGoToPage (gboolean sensitive) = 0;
-            
+
             ///
             /// @brief Changes the sensitivity of the "Go To Prev. Page" action.
             ///
@@ -183,7 +199,22 @@ namespace ePDFView
             ///                  insensitive (disable) it.
             ///
             virtual void sensitiveGoToPreviousPage (gboolean sensitive) = 0;
-            
+
+#if defined (HAVE_CUPS)
+            ///
+            /// @brief Changes the sensitivity of the "Print" action.
+            ///
+            /// The view must change the sensitivity (it's called enabled or
+            /// disabled on some toolkits) of the "Print" action
+            /// (both on the menu and the toolbar or any other place).
+            ///
+            /// @param sensitive Set to TRUE if need to make the action
+            ///                  sensitive (enable) or FALSE to
+            ///                  insensitive (disable) it.
+            ///
+            virtual void sensitivePrint (gboolean sensitive) = 0;
+#endif // HAVE_CUPS
+
             ///
             /// @brief Changes the sensitivity of the "Open" action.
             ///
@@ -209,7 +240,7 @@ namespace ePDFView
             ///                  insensitive (disable) it.
             ///
             virtual void sensitiveReload (gboolean sensitive) = 0;
-            
+
             ///
             /// @brief Changes the sensitivity of the "Rotate Left" action.
             ///
@@ -222,7 +253,7 @@ namespace ePDFView
             ///                  insensitive (disable) it.
             ///
             virtual void sensitiveRotateLeft (gboolean sensitive) = 0;
-            
+
             ///
             /// @brief Changes the sensitivity of the "Rotate Right" action.
             ///
@@ -235,7 +266,32 @@ namespace ePDFView
             ///                  insensitive (disable) it.
             ///
             virtual void sensitiveRotateRight (gboolean sensitive) = 0;
-            
+
+            ///
+            /// @brief Changes the sensitivity of the "Save a Copy" action.
+            ///
+            /// The view must change the sensitivity (it's called enabled or
+            /// disabled on some toolkits) of the "Save a Copy" action
+            /// (both on the menu and the toolbar or any other place).
+            ///
+            /// @param sensitive Set to TRUE if need to make sensitive (enable)
+            ///                  the action or FALSE to insensitive (disable)
+            ///                  it.
+            ///
+            virtual void sensitiveSave (gboolean sensitive) = 0;
+
+            ///
+            /// @brief Changes the sensitivity of the "Zoom" entry.
+            ///
+            /// The view must change the sensitivity (it's called enabled or
+            /// disabled on some toolkits) of the "Zoom" entry
+            /// on the toolbar.
+            ///
+            /// @param sensitive Set to TRUE if need to make sensitive (enable)
+            ///                  the entry or FALSE to insensitive (disable) it.
+            ///
+            virtual void sensitiveZoom (gboolean sensitive) = 0;
+
             ///
             /// @brief Changes the sensitivity of the "Zoom In" action.
             ///
@@ -248,7 +304,7 @@ namespace ePDFView
             ///                  insensitive (disable) it.
             ///
             virtual void sensitiveZoomIn (gboolean sensitive) = 0;
-            
+
             ///
             /// @brief Changes the sensitivity of the "Zoom Out" action.
             ///
@@ -261,7 +317,7 @@ namespace ePDFView
             ///                  insensitive (disable) it.
             ///
             virtual void sensitiveZoomOut (gboolean sensitive) = 0;
-            
+
             ///
             /// @brief Changes the sensitivity of the "Zoom to Fit" action.
             ///
@@ -274,7 +330,7 @@ namespace ePDFView
             ///                  insensitive (disable) it.
             ///
             virtual void sensitiveZoomFit (gboolean sensitive) = 0;
-            
+
             ///
             /// @brief Changes the sensitivity of the "Zoom to Width" action.
             ///
@@ -298,6 +354,17 @@ namespace ePDFView
             ///
             virtual void setCursor (ViewCursor cursorType) = 0;
 
+            ///
+            /// @brief Sets the text for the number of pages.
+            ///
+            /// The view displays the number of pages separately
+            /// of the current page. This function should set the
+            /// text to display in this separate control.
+            ///
+            /// @param text The text to set as the number of pages.
+            ///
+            virtual void setNumberOfPagesText (const gchar *text) = 0;
+
             ///
             /// @brief Sets the text of the status bar.
             ///
@@ -309,6 +376,17 @@ namespace ePDFView
             ///
             virtual void setStatusBarText (const gchar *text) = 0;
 
+            ///
+            /// @brief Sets the current zoom text.
+            ///
+            /// The current zoom text is displayed on the tool bar and
+            /// the user can change it in order to specify a concrete
+            /// zoom level.
+            ///
+            /// @param text The text to show in the current zoom entry.
+            ///
+            virtual void setZoomText (const gchar *text) = 0;
+
             ///
             /// @brief Makes the main view to show itself.
             ///
@@ -330,9 +408,9 @@ namespace ePDFView
             /// @param title The error dialog's title.
             /// @param body The error dialog's body (the error message.)
             ///
-            virtual void showErrorMessage (const gchar *title, 
+            virtual void showErrorMessage (const gchar *title,
                                            const gchar *body) = 0;
-            
+
             ///
             /// @brief Shows the document's index.
             ///
@@ -358,7 +436,7 @@ namespace ePDFView
             ///             FALSE otherwise.
             ///
             virtual void showStatusbar (gboolean show) = 0;
-            
+
             ///
             /// @brief Shows the tool bar.
             ///
@@ -368,16 +446,6 @@ namespace ePDFView
             /// @param show TRUE if the toolbar must be shown, FALSE otherwise.
             ///
             virtual void showToolbar (gboolean show) = 0;
-            
-            ///
-            /// @brief Gives the number of pages that the document has.
-            ///
-            /// The view should keep a the number of total pages and show it
-            /// to the user.
-            ///
-            /// @param pages The total number of pages the document has.
-            ///
-            virtual void setTotalPages (gint pages) = 0;
 
             ///
             /// @brief Sets the "Go To Page" text.
@@ -416,7 +484,7 @@ namespace ePDFView
             /// @return The find's view. The returned pointer will be
             ///         freed by the main view.
             virtual IFindView *getFindView (void) = 0;
-            
+
             ///
             /// @brief Gets the page's view.
             ///
@@ -432,6 +500,40 @@ namespace ePDFView
             ///
             virtual IPageView *getPageView (void) = 0;
 
+            ///
+            /// @brief Gets the preferences' view.
+            ///
+            /// The preferences view is created each time the presenter
+            /// calls this function, with the main view as its parent window.
+            ///
+            /// @return The new preferences view. The returned pointer
+            ///         must be freed by the caller.
+            ///
+            virtual IPreferencesView *getPreferencesView (void) = 0;
+
+#if defined (HAVE_CUPS)
+            ///
+            /// @brief Gets the print's view.
+            ///
+            /// The print view is created each time the presenter
+            /// class this function, with the main view as its parent window.
+            ///
+            /// @return The new print view. The returned pointer must be
+            ///         freed by the caller.
+            ///
+            virtual IPrintView *getPrintView (void) = 0;
+#endif // HAVE_CUPS
+
+            ///
+            /// @brief Gets the text from the zoom entry.
+            ///
+            /// Gets the text inside the "Current Zoom" entry on the
+            /// tool bar.
+            ///
+            /// @return The text inside the "Current Zoom" entry.
+            ///
+            virtual const gchar *getZoomText (void) = 0;
+
             ///
             /// @brief Sets the main view's title.
             ///
