@@ -25,6 +25,8 @@ using namespace ePDFView;
 #if defined (DEBUG)
 G_LOCK_DEFINE_STATIC (fileLoaded);
 static volatile gboolean fileLoaded;
+G_LOCK_DEFINE_STATIC (fileSaved);
+static volatile gboolean fileSaved;
 #endif // DEBUG
 
 ///
@@ -55,6 +57,9 @@ MainPter::MainPter (IDocument *document)
     G_LOCK (fileLoaded);
     fileLoaded = FALSE;
     G_UNLOCK (fileLoaded);
+    G_LOCK (fileSaved);
+    fileSaved = FALSE;
+    G_UNLOCK (fileSaved);
 #endif
 }
 
@@ -103,7 +108,7 @@ void
 MainPter::setInitialState ()
 {
     g_assert ( NULL != m_Document && "The document is NULL.");
-    
+
     IMainView &view = getView ();
     Config &config = Config::getConfig ();
     gboolean showSidebar = FALSE;
@@ -117,18 +122,23 @@ MainPter::setInitialState ()
         {
             view.setTitle (m_Document->getTitle ());
         }
+        setZoomText (m_Document->getZoom ());
         view.sensitiveFind (TRUE);
         view.sensitiveGoToPage (TRUE);
         view.sensitiveReload (TRUE);
         view.sensitiveRotateRight (TRUE);
         view.sensitiveRotateLeft (TRUE);
+        view.sensitiveSave (TRUE);
+        view.sensitiveZoom (TRUE);
         view.sensitiveZoomIn (TRUE);
         view.sensitiveZoomOut (TRUE);
         view.sensitiveZoomFit (TRUE);
         view.sensitiveZoomWidth (TRUE);
         view.activeZoomFit (config.zoomToFit ());
         view.activeZoomWidth (config.zoomToWidth ());
-        view.setTotalPages (m_Document->getNumPages ());
+#if defined (HAVE_CUPS)
+        view.sensitivePrint (TRUE);
+#endif // HAVE_CUPS
 
         checkZoomSettings ();
 
@@ -136,11 +146,11 @@ MainPter::setInitialState ()
         showSidebar = (NULL != m_Document->getOutline () &&
                        0 < m_Document->getOutline ()->getNumChildren () &&
                        PageModeOutlines == m_Document->getPageMode ());
-    
     }
     else
     {
         view.setTitle (_("PDF Viewer"));
+        setZoomText (1.0);
         view.sensitiveFind (FALSE);
         view.sensitiveGoToFirstPage (FALSE);
         view.sensitiveGoToLastPage (FALSE);
@@ -150,11 +160,16 @@ MainPter::setInitialState ()
         view.sensitiveReload (FALSE);
         view.sensitiveRotateRight (FALSE);
         view.sensitiveRotateLeft (FALSE);
+        view.sensitiveSave (FALSE);
+        view.sensitiveZoom (FALSE);
         view.sensitiveZoomIn (FALSE);
         view.sensitiveZoomOut (FALSE);
         view.sensitiveZoomFit (FALSE);
         view.sensitiveZoomWidth (FALSE);
         showSidebar = FALSE;
+#if defined (HAVE_CUPS)
+        view.sensitivePrint (FALSE);
+#endif // HAVE_CUPS
     }
 
     // Sensitive the open file action.
@@ -173,7 +188,6 @@ MainPter::setInitialState ()
     // to be the first page. If we don't do this, the first page pointed
     // by the first outline is selected and we don't want this.
     view.show ();
-    
 }
 
 ///
@@ -190,6 +204,7 @@ void
 MainPter::setOpenState (const gchar *fileName, gboolean reload)
 {
     // Insensitive the open, reload, page navigation, zoom and rotate.
+    setZoomText (1.0);
     IMainView &view = getView ();
     view.sensitiveFind (FALSE);
     view.sensitiveGoToFirstPage (FALSE);
@@ -200,11 +215,16 @@ MainPter::setOpenState (const gchar *fileName, gboolean reload)
     view.sensitiveReload (FALSE);
     view.sensitiveRotateRight (FALSE);
     view.sensitiveRotateLeft (FALSE);
+    view.sensitiveSave (FALSE);
+    view.sensitiveZoom (FALSE);
     view.sensitiveZoomIn (FALSE);
     view.sensitiveZoomOut (FALSE);
     view.sensitiveZoomFit (FALSE);
     view.sensitiveZoomWidth (FALSE);
     view.sensitiveOpen (FALSE);
+#if defined (HAVE_CUPS)
+    view.sensitivePrint (FALSE);
+#endif // HAVE_CUPS
     // Show the text on the status bar.
     gchar *statusText = NULL;
     if ( reload )
@@ -223,6 +243,19 @@ MainPter::setOpenState (const gchar *fileName, gboolean reload)
     m_PasswordTries = 3;
 }
 
+///
+/// @brief Tells the view which text to show in the zoom entry.
+///
+/// @param zoom The current document's zoom value.
+///
+void
+MainPter::setZoomText (gdouble zoom)
+{
+    gchar *zoomText = g_strdup_printf ("%.3g", zoom * 100.0);
+    getView ().setZoomText (zoomText);
+    g_free (zoomText);
+}
+
 ///
 /// @brief Gets the presenter's view.
 ///
@@ -293,7 +326,7 @@ MainPter::goToLastPageActivated ()
 {
     g_assert (NULL != m_Document && 
               "Tried to go to the last page of a NULL document.");
-    
+
     m_PagePter->setNextPageScroll (PAGE_SCROLL_START);
     m_Document->goToLastPage ();
 }
@@ -306,7 +339,7 @@ MainPter::goToNextPageActivated ()
 {
     g_assert (NULL != m_Document && 
               "Tried to go to the next page of a NULL document.");
-   
+
     if ( m_Document->getCurrentPageNum () < m_Document->getNumPages () )
     {
         m_PagePter->setNextPageScroll (PAGE_SCROLL_START);
@@ -324,9 +357,9 @@ MainPter::goToNextPageActivated ()
 void
 MainPter::goToPageActivated ()
 {
-    g_assert (NULL != m_Document && 
+    g_assert (NULL != m_Document &&
               "Tried to go to the a page of a NULL document.");
-    
+
     // First try to get the page number from the view.
     const gchar *goToPageText = getView ().getGoToPageText ();
     if ( NULL != goToPageText )
@@ -341,11 +374,12 @@ MainPter::goToPageActivated ()
         // Set the text for the current page.
         gint newPageNum = m_Document->getCurrentPageNum ();
         gint totalPages = m_Document->getNumPages ();
-        gchar *goToPageText = g_strdup_printf (_("%d of %d"), 
-                                               newPageNum, totalPages);
+        gchar *goToPageText = g_strdup_printf ("%d", newPageNum);
         getView ().setGoToPageText (goToPageText);
         g_free (goToPageText);
-    
+        gchar *totalPagesText = g_strdup_printf (_("of %d"), totalPages);
+        getView ().setNumberOfPagesText (totalPagesText);
+        g_free (totalPagesText);
     }
 }
 
@@ -358,7 +392,7 @@ MainPter::goToPreviousPageActivated ()
 {
     g_assert (NULL != m_Document && 
               "Tried to go to the previous page of a NULL document.");
-   
+
     if ( 1 < m_Document->getCurrentPageNum () )
     {
         m_PagePter->setNextPageScroll (PAGE_SCROLL_END);
@@ -382,15 +416,16 @@ MainPter::openFileActivated ()
     gchar *lastFolder = config.getOpenFileFolder ();
     IMainView &view = getView ();
     gchar *fileName = view.openFileDialog (lastFolder);
-    g_free (lastFolder);    
+    g_free (lastFolder);
     if ( NULL != fileName )
     {
-        gchar *dirName = g_path_get_dirname (fileName);        
+        gchar *dirName = g_path_get_dirname (fileName);
         config.setOpenFileFolder (dirName);
         g_free (dirName);
         // Open the file.
         setOpenState (fileName, FALSE);
         m_Document->load (fileName, NULL);
+        g_free (fileName);
     }
 }
 
@@ -406,11 +441,54 @@ void
 MainPter::outlineActivated (DocumentOutline *outline)
 {
     g_assert (NULL != outline && "The outline activated is NULL.");
-    
+
     m_PagePter->setNextPageScroll (PAGE_SCROLL_START);
     m_Document->goToPage (outline->getDestinationPage ());
 }
 
+#if defined (HAVE_CUPS)
+///
+/// @brief The "print" action was activated.
+///
+void
+MainPter::printActivated ()
+{
+    PrintPter *print = new PrintPter (m_Document);
+    IPrintView *view = getView ().getPrintView ();
+    if ( NULL != view )
+    {
+        print->setView (view);
+    }
+    else
+    {
+        delete print;
+    }
+    // There's no need to keep a pointer to the presenter because
+    // it will destroy itself when the dialog is closed.
+}
+#endif // HAVE_CUPS
+
+///
+/// @brief The "Preferences" was activated.
+///
+void
+MainPter::preferencesActivated ()
+{
+    PreferencesPter *preferences = new PreferencesPter ();
+    IPreferencesView *view = getView ().getPreferencesView ();
+    if ( NULL != view )
+    {
+        preferences->setView (view);
+    }
+    else
+    {
+        delete preferences;
+    }
+    // There's no need to keep a pointer of the preferences presenter because
+    // when the dialog is closed or the application is finished it will
+    // destroy itself.
+}
+
 ///
 /// @brief The "Reload" was activated.
 ///
@@ -419,7 +497,7 @@ MainPter::outlineActivated (DocumentOutline *outline)
 ///
 void
 MainPter::reloadActivated ()
-{   
+{
     g_assert ( m_Document->isLoaded () && 
             "Tried to reload a yet to load document.");
     // Reload
@@ -451,6 +529,41 @@ MainPter::rotateRightActivated ()
     checkZoomSettings ();
 }
 
+///
+/// @brief The Save File action was activated.
+///
+/// This means that the user wants to save a copy of the document.
+/// The presenter asks the view to show the Save File dialog and return a 
+/// file name.
+/// Then it will try to save the document and if there's any error it will 
+/// request the view to show an error dialog.
+///
+void
+MainPter::saveFileActivated ()
+{
+    Config &config = Config::getConfig ();
+    gchar *lastFolder = config.getSaveFileFolder ();
+    IMainView &view = getView ();
+    gchar *fileName = view.saveFileDialog (lastFolder);
+    g_free (lastFolder);
+    if ( NULL != fileName )
+    {
+
+        // Show on the status bar that we are saving the copy.
+        gchar *statusText = g_strdup_printf (_("Saving document to %s..."),
+                                             fileName);
+        view.setStatusBarText (statusText);
+        g_free (statusText);
+        gchar *dirName = g_path_get_dirname (fileName);
+        config.setSaveFileFolder (dirName);
+        g_free (dirName);
+        // Save the document.
+        m_Document->save (fileName);
+        g_free (fileName);
+    }
+}
+
+
 ///
 /// @brief The "Show Index" was activated.
 ///
@@ -486,6 +599,28 @@ MainPter::showToolbarActivated (gboolean show)
     getView ().showToolbar (show);
 }
 
+///
+/// @brief The user entered a zoom value.
+///
+void
+MainPter::zoomActivated ()
+{
+    IMainView &view = getView ();
+    const gchar *zoomText = view.getZoomText ();
+    if ( NULL != zoomText )
+    {
+        Config &config = Config::getConfig ();
+        config.setZoomToFit (FALSE);
+        config.setZoomToWidth (FALSE);
+
+        view.activeZoomFit (FALSE);
+        view.activeZoomWidth (FALSE);
+
+        gdouble zoom = atof (zoomText);
+        m_Document->setZoom (zoom / 100.0);
+    }
+}
+
 ///
 /// @brief Performs the "Zoom Fit Best".
 ///
@@ -512,14 +647,17 @@ MainPter::zoomFitActivated (gboolean active)
     g_assert ( NULL != m_Document && "Tried to zoom fit a NULL document.");
 
     Config &config = Config::getConfig ();
-    config.setZoomToFit (active);
     IMainView &view = getView ();
-    view.activeZoomFit (config.zoomToFit ());
-    view.activeZoomWidth (config.zoomToWidth ());
-    if ( active )
+    if ( active && !config.zoomToFit () )
     {
+        config.setZoomToFit (TRUE);
+        view.activeZoomWidth (FALSE);
         zoomFit ();
     }
+    else if ( config.zoomToFit () )
+    {
+        view.activeZoomFit (TRUE);
+    }
 }
 
 ///
@@ -537,7 +675,7 @@ MainPter::zoomInActivated ()
     IMainView &view = getView ();
     view.activeZoomFit (FALSE);
     view.activeZoomWidth (FALSE);
-    
+
     m_Document->zoomIn ();
 }
 
@@ -556,7 +694,7 @@ MainPter::zoomOutActivated ()
     IMainView &view = getView ();
     view.activeZoomFit (FALSE);
     view.activeZoomWidth (FALSE);
-   
+
     m_Document->zoomOut ();
 }
 
@@ -578,7 +716,8 @@ MainPter::zoomWidth ()
 ///
 /// @brief The "Zoom Fit Width" was activated.
 ///
-/// @param active TRUE if the zoom to width option is activated, false otherwise.
+/// @param active TRUE if the zoom to width option is activated,
+///               FALSE otherwise.
 ///
 void
 MainPter::zoomWidthActivated (gboolean active)
@@ -586,14 +725,18 @@ MainPter::zoomWidthActivated (gboolean active)
     g_assert ( NULL != m_Document && "Tried to zoom width a NULL document.");
 
     Config &config = Config::getConfig ();
-    config.setZoomToWidth (active);
     IMainView &view = getView ();
-    view.activeZoomFit (config.zoomToFit ());
-    view.activeZoomWidth (config.zoomToWidth ());
-    if ( active )
+    if ( active && !config.zoomToWidth () )
     {
+        config.setZoomToWidth (TRUE);
+        view.activeZoomFit (FALSE);
+        view.activeZoomWidth (TRUE);
         zoomWidth ();
     }
+    else if ( config.zoomToWidth () )
+    {
+        view.activeZoomWidth (TRUE);
+    }
 }
 
 ///
@@ -702,10 +845,13 @@ MainPter::notifyPageChanged (gint pageNum)
     IMainView &view = getView ();
     // Set the text for the current page.
     gint totalPages = m_Document->getNumPages ();
-    gchar *goToPageText = g_strdup_printf (_("%d of %d"), pageNum, totalPages);
+    gchar *goToPageText = g_strdup_printf ("%d", pageNum);
     view.setGoToPageText (goToPageText);
     g_free (goToPageText);
-    
+    gchar *totalPagesText = g_strdup_printf(_("of %d"), totalPages);
+    view.setNumberOfPagesText (totalPagesText);
+    g_free (totalPagesText);
+
     // Set the page navigation sensitivity.
     gboolean documentLoaded = m_Document->isLoaded ();
     gint numPages = m_Document->getNumPages ();
@@ -730,6 +876,8 @@ void
 MainPter::notifyPageZoomed (gdouble zoom)
 {
     IMainView &view = getView ();
+    // Show the new zoom.
+    setZoomText (zoom);
     // Set the zoom sensitivity.
     view.sensitiveZoomIn (m_Document->canZoomIn ());
     view.sensitiveZoomOut (m_Document->canZoomOut ());
@@ -746,6 +894,29 @@ MainPter::notifyReload ()
 #endif // DEBUG
 }
 
+void
+MainPter::notifySave ()
+{
+    // Remove the status text.
+    getView ().setStatusBarText (NULL);
+#if defined (DEBUG)
+    G_LOCK (fileSaved);
+    fileSaved = TRUE;
+    G_UNLOCK (fileSaved);
+#endif // DEBUG
+}
+
+void
+MainPter::notifySaveError (const GError *error)
+{
+    getView ().showErrorMessage (_("Error Saving File"), error->message);
+#if defined (DEBUG)
+    G_LOCK (fileSaved);
+    fileSaved = TRUE;
+    G_UNLOCK (fileSaved);
+#endif // DEBUG
+}
+
 #if defined (DEBUG)
 ///
 /// @brief Waits until a file is loaded.
@@ -767,4 +938,25 @@ MainPter::waitForFileLoaded ()
     fileLoaded = FALSE;
     G_UNLOCK (fileLoaded);
 }
+
+///
+/// @brief Waits until a file is saved.
+///
+/// This is *only* useful for testing, don't use it.
+///
+void
+MainPter::waitForFileSaved ()
+{
+    volatile gboolean end = FALSE;
+    do
+    {
+        G_LOCK (fileSaved);
+        end = fileSaved;
+        G_UNLOCK (fileSaved);
+    }
+    while ( !end );
+    G_LOCK (fileSaved);
+    fileSaved = FALSE;
+    G_UNLOCK (fileSaved);
+}
 #endif // DEBUG
