@@ -17,7 +17,7 @@
 
 #include <config.h>
 #include <time.h>
-#include <glib/poppler.h>
+#include <poppler.h>
 #include "epdfview.h"
 
 using namespace ePDFView;
@@ -38,6 +38,7 @@ PDFDocument::PDFDocument ():
     IDocument ()
 {
     m_Document = NULL;
+    m_PostScript = NULL;
 }
 
 ///
@@ -46,6 +47,7 @@ PDFDocument::PDFDocument ():
 PDFDocument::~PDFDocument ()
 {
     clearCache ();
+    outputPostscriptEnd ();
     if ( NULL != m_Document )
     {
         g_object_unref (G_OBJECT (m_Document));
@@ -53,13 +55,95 @@ PDFDocument::~PDFDocument ()
     }
 }
 
+IDocument *
+PDFDocument::copy () const
+{
+    // Making a deep copy is just making a new document and loading the same
+    // file.
+    PDFDocument *newDocument = new PDFDocument ();
+    newDocument->loadFile (getFileName (), getPassword (), NULL);
+
+    return newDocument;
+}
+
+///
+/// @brief Creates a new document link.
+///
+/// Based on the passed @a link, this function creates a new document
+/// links of the best type for the link's action.
+///
+/// @param link The link to create the document link from.
+/// @param pageHeight The unscaled size of the page's height.
+/// @param scale The scale to calculate the link's coordinates.
+///
+/// @return The link best suited for @a link or NULL if no link can be
+///         created.
+IDocumentLink *
+PDFDocument::createDocumentLink (const PopplerLinkMapping *link,
+                                 const gdouble pageHeight,
+                                 const gdouble scale)
+{
+    PopplerAction *action = link->action;
+    IDocumentLink *documentLink = NULL;
+
+    // Calculate the four link's corners.
+    gdouble topLeft = link->area.x1 * scale;
+    gdouble topRight = link->area.x2 * scale;
+    gdouble bottomLeft = (pageHeight - link->area.y2) * scale;
+    gdouble bottomRight = (pageHeight - link->area.y1) * scale;
+
+    switch (action->type)
+    {
+        // Internal cross-reference link.
+        case POPPLER_ACTION_GOTO_DEST:
+        {
+            PopplerActionGotoDest *actionGoTo = (PopplerActionGotoDest *)action;
+            PopplerDest *destination = actionGoTo->dest;
+            int pageNum = destination->page_num;
+#if defined (HAVE_POPPLER_0_5_2)
+            if ( POPPLER_DEST_NAMED == destination->type )
+            {
+                destination =
+                    poppler_document_find_dest (m_Document,
+                                                destination->named_dest);
+                if ( NULL != destination )
+                {
+                    pageNum = destination->page_num;
+                    poppler_dest_free (destination);
+                }
+            }
+#endif // HAVE_POPPLER_0_5_2
+
+            documentLink = new DocumentLinkGoto (
+                    topLeft, bottomLeft, topRight, bottomRight,
+                    pageNum);
+            break;
+        }
+        // Internet address.
+        case POPPLER_ACTION_URI:
+        {
+            PopplerActionUri *actionUri = (PopplerActionUri *)action;
+            documentLink = new DocumentLinkUri (
+                    topLeft, bottomLeft, topRight, bottomRight,
+                    actionUri->uri);
+            break;
+        }
+
+        default:
+            g_warning ("Poppler's link type %d not handled.", action->type);
+            break;
+    }
+
+    return documentLink;
+}
+
 GList *
 PDFDocument::findTextInPage (gint pageNum, const gchar *textToFind)
 {
     GList *results = NULL;
-    
+
     if ( NULL == m_Document )
-    {    
+    {
         return results;
     }
 
@@ -125,7 +209,7 @@ PDFDocument::loadFile (const gchar *filename, const gchar *password,
     }
     // Try to open the PDF document.
     GError *loadError = NULL;
-    PopplerDocument *newDocument = 
+    PopplerDocument *newDocument =
         poppler_document_new_from_file (filename_uri, password, &loadError);
     g_free (filename_uri);
     // Check if the document couldn't be opened successfully and why.
@@ -160,10 +244,10 @@ PDFDocument::loadFile (const gchar *filename, const gchar *password,
 
         return FALSE;
     }
-   
+
     // Set the used filename and password to let the user reload the
     // document.
-    setFileName (filename);    
+    setFileName (filename);
     setPassword (password);
     if ( NULL != m_Document )
     {
@@ -206,7 +290,7 @@ PDFDocument::loadMetadata (void)
     gchar *producer = NULL;
     gchar *subject = NULL;
     gchar *title = NULL;
-   
+
     g_object_get (m_Document,
             "author", &author,
             "creation-date", &creationDate,
@@ -288,17 +372,9 @@ PDFDocument::setLinks (DocumentPage *renderedPage, PopplerPage *popplerPage)
          pageLink = g_list_next (pageLink) )
     {
         PopplerLinkMapping *link = (PopplerLinkMapping *)pageLink->data;
-        PopplerAction *action = link->action;
-        // Only internal links.
-        if ( POPPLER_ACTION_GOTO_DEST == action->type )
+        IDocumentLink *documentLink = createDocumentLink (link, pageHeight, scale);
+        if ( NULL != documentLink )
         {
-            PopplerActionGotoDest *actionGoTo = (PopplerActionGotoDest *)action;
-            DocumentLink *documentLink = 
-                new DocumentLink (link->area.x1 * scale, 
-                                  (pageHeight - link->area.y2) * scale,
-                                  link->area.x2 * scale,
-                                  (pageHeight - link->area.y1) * scale,
-                                  actionGoTo->dest->page_num);
             renderedPage->addLink (documentLink);
         }
     }
@@ -329,14 +405,29 @@ PDFDocument::setOutline (DocumentOutline *outline,
                 poppler_index_iter_get_action (childrenList);
             if ( POPPLER_ACTION_GOTO_DEST == action->type )
             {
-                PopplerActionGotoDest *actionGoTo = 
+                PopplerActionGotoDest *actionGoTo =
                     (PopplerActionGotoDest *)action;
                 DocumentOutline *child = new DocumentOutline ();
                 child->setParent (outline);
                 child->setTitle (actionGoTo->title);
                 PopplerDest *destination = actionGoTo->dest;
                 child->setDestination (destination->page_num);
-                outline->addChild (child);
+#if defined (HAVE_POPPLER_0_5_2)
+                if ( POPPLER_DEST_NAMED == destination->type )
+                {
+                    destination =
+                        poppler_document_find_dest (m_Document,
+                                                    destination->named_dest);
+                    if ( NULL != destination )
+                    {
+                        child->setDestination (destination->page_num);
+                        poppler_dest_free (destination);
+                        destination = NULL;
+                    }
+                }
+#endif // HAVE_POPPLER_0_5_2
+
+                    outline->addChild (child);
                 PopplerIndexIter *childIter = 
                     poppler_index_iter_get_child (childrenList);
                 setOutline (child, childIter);
@@ -366,27 +457,79 @@ PDFDocument::getPageSizeForPage (gint pageNum, gdouble *width, gdouble *height)
     g_assert (NULL != height && "Tried to save the page's height to NULL.");
 
     PopplerPage *page = poppler_document_get_page (m_Document, pageNum - 1);
-
-    gdouble pageWidth;
-    gdouble pageHeight;
-    // Check which rotation has the document's page to know what is width
-    // and what is height.
-    gint rotate = getRotation ();
-    if ( 90 == rotate || 270 == rotate )
+    if ( NULL != page )
     {
-        poppler_page_get_size (page, &pageHeight, &pageWidth);
+        gdouble pageWidth;
+        gdouble pageHeight;
+        // Check which rotation has the document's page to know what is width
+        // and what is height.
+        gint rotate = getRotation ();
+        if ( 90 == rotate || 270 == rotate )
+        {
+            poppler_page_get_size (page, &pageHeight, &pageWidth);
+        }
+        else 
+        {
+            poppler_page_get_size (page, &pageWidth, &pageHeight);
+        }
+
+        *width = pageWidth;
+        *height = pageHeight;
+
+        g_object_unref (G_OBJECT (page));
     }
-    else 
+    else
     {
-        poppler_page_get_size (page, &pageWidth, &pageHeight);
+        // Just give any valor, except 0.
+        *width = 1.0;
+        *height = 1.0;
     }
-
-    *width = pageWidth;
-    *height = pageHeight;
-
-    g_object_unref (G_OBJECT (page));
 }
 
+void
+PDFDocument::outputPostscriptBegin (const gchar *fileName, guint numOfPages,
+                                    gfloat pageWidth, gfloat pageHeight)
+{
+    if ( NULL != m_Document )
+    {
+        if ( NULL != m_PostScript )
+        {
+            outputPostscriptEnd ();
+        }
+        m_PostScript =
+            poppler_ps_file_new (m_Document, fileName, 0, numOfPages);
+        if ( NULL != m_PostScript )
+        {
+            poppler_ps_file_set_paper_size (m_PostScript,
+                                            pageWidth, pageHeight);
+        }
+    }
+}
+
+void
+PDFDocument::outputPostscriptEnd ()
+{
+    if ( NULL != m_PostScript )
+    {
+        poppler_ps_file_free (m_PostScript);
+        m_PostScript = NULL;
+    }
+}
+
+void
+PDFDocument::outputPostscriptPage (guint pageNum)
+{
+    if ( NULL != m_Document && NULL != m_PostScript )
+    {
+        PopplerPage *page = poppler_document_get_page (m_Document, pageNum - 1);
+        if ( NULL != page )
+        {
+            poppler_page_render_to_ps (page, m_PostScript);
+        }
+    }
+}
+
+
 ///
 /// @brief Renders a document's page.
 ///
@@ -405,7 +548,7 @@ PDFDocument::renderPage (gint pageNum)
     {
         return NULL;
     }
-    
+
     // First create the document's page.
     gdouble pageWidth;
     gdouble pageHeight;
@@ -424,15 +567,50 @@ PDFDocument::renderPage (gint pageNum)
                                                   renderedPage->getRowStride (),
                                                   NULL, NULL);
     PopplerPage *page = poppler_document_get_page (m_Document, pageNum - 1);
-    poppler_page_render_to_pixbuf (page, 0, 0, width, height, getZoom (),
-                                   getRotation (), pixbuf);
+    if ( NULL != page )
+    {
+        poppler_page_render_to_pixbuf (page, 0, 0, width, height, getZoom (),
+                                       getRotation (), pixbuf);
+        setLinks (renderedPage, page);
+        g_object_unref (G_OBJECT (page));
+    }
     g_object_unref (pixbuf);
-    setLinks (renderedPage, page);
-    g_object_unref (G_OBJECT (page));
-    
+
     return (renderedPage);
 }
 
+///
+/// @brief Saves a document's copy to a file.
+///
+/// Tries to save the document to file @a filename.
+///
+/// @param filename The path, absolute or relative, to the file name
+///                 to save the copy to.
+/// @param error Location to store any error that could happen or
+///              set to NULL to ignore errors.
+///
+/// @return TRUE if the file could be saved. FALSE otherwise.
+///
+gboolean
+PDFDocument::saveFile (const gchar *filename, GError **error)
+{
+    g_assert (NULL != m_Document && "No document loaded yet.");
+    g_assert (NULL != filename && "Tried to save to a NULL file name.");
+
+    gchar *absoluteFileName = getAbsoluteFileName (filename);
+    gchar *filename_uri = g_filename_to_uri (absoluteFileName, NULL, error);
+    g_free (absoluteFileName);
+    if ( NULL == filename_uri )
+    {
+        return FALSE;
+    }
+    // Try to save the PDF document.
+    gboolean result = poppler_document_save (m_Document, filename_uri, error);
+    g_free (filename_uri);
+
+    return result;
+}
+
 ///
 /// @brief Gets the document's page layout from Poppler's page layout.
 ///
