@@ -24,26 +24,33 @@
 #include "StockIcons.h"
 #include "FindView.h"
 #include "PageView.h"
+#include "PreferencesView.h"
+#if defined (HAVE_CUPS)
+#include "PrintView.h"
+#endif // HAVE_CUPS
 #include "MainView.h"
 
 using namespace ePDFView;
 
 // Constants.
 static gint CURRENT_PAGE_POS = 5;
+static gint CURRENT_PAGE_WIDTH = 5;
+static gint CURRENT_ZOOM_POS = 8;
+static gint CURRENT_ZOOM_WIDTH = 6;
 
 // Enumerations.
-static enum 
+enum indexColumns
 {
     outlineTitleColumn,
     outlinePageColumn,
     outlineDataColumn,
     outlineNumColumns
-} indexColumns;
+};
 
 // Forward declarations.
 static void main_window_about_box_cb (GtkWidget *, gpointer);
 static void main_window_find_cb (GtkWidget *, gpointer);
-static gboolean main_window_moved_or_resized_cb (GtkWidget *, 
+static gboolean main_window_moved_or_resized_cb (GtkWidget *,
                                                  GdkEventConfigure *, gpointer);
 static void main_window_go_to_first_page_cb (GtkWidget *, gpointer);
 static void main_window_go_to_last_page_cb (GtkWidget *, gpointer);
@@ -55,15 +62,22 @@ static void main_window_rotate_left_cb (GtkWidget *, gpointer);
 static void main_window_rotate_right_cb (GtkWidget *, gpointer);
 static void main_window_open_file_cb (GtkWidget *, gpointer);
 static void main_window_outline_cb (GtkTreeSelection *, gpointer);
+static void main_window_preferences_cb (GtkWidget *, gpointer);
 static void main_window_quit_cb (GtkWidget *, gpointer);
+static void main_window_save_file_cb (GtkWidget *, gpointer);
 static void main_window_show_index_cb (GtkToggleAction *, gpointer);
 static void main_window_show_statusbar_cb (GtkToggleAction *, gpointer);
 static void main_window_show_toolbar_cb (GtkToggleAction *, gpointer);
+static void main_window_zoom_cb (GtkWidget *, gpointer);
 static void main_window_zoom_fit_cb (GtkToggleAction *, gpointer);
 static void main_window_zoom_in_cb (GtkWidget *, gpointer);
 static void main_window_zoom_out_cb (GtkWidget *, gpointer);
 static void main_window_zoom_width_cb (GtkToggleAction *, gpointer);
 
+#if defined (HAVE_CUPS)
+static void main_window_print_cb (GtkWidget *, gpointer);
+#endif // HAVE_CUPS
+
 // The actions for menus and toolbars.
 static const GtkActionEntry g_NormalEntries[] = 
 {
@@ -73,47 +87,61 @@ static const GtkActionEntry g_NormalEntries[] =
     { "GoMenu", NULL, N_("_Go"), NULL, NULL, NULL },
     { "HelpMenu", NULL, N_("_Help"), NULL, NULL, NULL },
 
-    { "OpenFile", GTK_STOCK_OPEN, N_("_Open"), "<control>O",       
+    { "OpenFile", GTK_STOCK_OPEN, N_("_Open"), "<control>O",
       N_("Open a PDF document"),
       G_CALLBACK (main_window_open_file_cb) },
 
     { "ReloadFile", GTK_STOCK_REFRESH, N_("_Reload"), "<control>R",
       N_("Reload the current document"),
       G_CALLBACK (main_window_reload_cb) },
-    
-    { "Quit", GTK_STOCK_CLOSE, N_("_Close"), "<control>W", 
-      N_("Close this window"), 
+
+    { "SaveFile", GTK_STOCK_SAVE, N_("_Save a Copy..."), "<control>S",
+      N_("Save a copy of the current document"),
+      G_CALLBACK (main_window_save_file_cb) },
+
+#if defined (HAVE_CUPS)
+    { "Print", GTK_STOCK_PRINT, N_("_Print..."), "<control>P",
+      N_("Print the current document"),
+      G_CALLBACK (main_window_print_cb) },
+#endif // HAVE_CUPS
+
+    { "Quit", GTK_STOCK_CLOSE, N_("_Close"), "<control>W",
+      N_("Close this window"),
       G_CALLBACK (main_window_quit_cb) },
 
     { "Find", GTK_STOCK_FIND, N_("_Find"), "<control>F",
       N_("Find a word in the document"),
       G_CALLBACK (main_window_find_cb) },
-    
-    { "ZoomIn", GTK_STOCK_ZOOM_IN, N_("Zoom _In"), "<control>plus", 
-      N_("Enlarge the document"), 
+
+    { "Preferences", GTK_STOCK_PREFERENCES, N_("Preferences..."), NULL,
+      N_("Change the application's preferences"),
+      G_CALLBACK (main_window_preferences_cb) },
+
+    { "ZoomIn", GTK_STOCK_ZOOM_IN, N_("Zoom _In"), "<control>plus",
+      N_("Enlarge the document"),
       G_CALLBACK (main_window_zoom_in_cb) },
 
-    { "ZoomOut", GTK_STOCK_ZOOM_OUT, N_("Zoom _Out"), "<control>minus", 
-      N_("Shrink the document"), 
+    { "ZoomOut", GTK_STOCK_ZOOM_OUT, N_("Zoom _Out"), "<control>minus",
+      N_("Shrink the document"),
       G_CALLBACK (main_window_zoom_out_cb) },
-    
-    { "RotateRight", EPDFVIEW_STOCK_ROTATE_RIGHT, N_("Rotate _Right"), NULL, 
-      N_("Rotate the document 90 degrees clockwise"), 
+
+    { "RotateRight", EPDFVIEW_STOCK_ROTATE_RIGHT, N_("Rotate _Right"), NULL,
+      N_("Rotate the document 90 degrees clockwise"),
       G_CALLBACK (main_window_rotate_right_cb) },
 
-    { "RotateLeft", EPDFVIEW_STOCK_ROTATE_LEFT, N_("Rotate _Left"), NULL, 
-      N_("Rotate the document 90 degrees counter-clockwise"), 
+    { "RotateLeft", EPDFVIEW_STOCK_ROTATE_LEFT, N_("Rotate _Left"), NULL,
+      N_("Rotate the document 90 degrees counter-clockwise"),
       G_CALLBACK (main_window_rotate_left_cb) },
 
     { "GoToFirstPage", GTK_STOCK_GOTO_FIRST, N_("_First Page"), "<control>Home",
-      N_("Go to the first page"), 
+      N_("Go to the first page"),
       G_CALLBACK (main_window_go_to_first_page_cb) },
 
-    { "GoToNextPage", GTK_STOCK_GO_FORWARD, N_("_Next Page"), "Page_Down", 
-      N_("Go to the next page"), 
+    { "GoToNextPage", GTK_STOCK_GO_FORWARD, N_("_Next Page"), "<Shift>Page_Down",
+      N_("Go to the next page"),
       G_CALLBACK (main_window_go_to_next_page_cb) },
 
-    { "GoToPreviousPage", GTK_STOCK_GO_BACK, N_("_Previous Page"), "Page_Up", 
+    { "GoToPreviousPage", GTK_STOCK_GO_BACK, N_("_Previous Page"), "<Shift>Page_Up",
       N_("Go to the previous page"),
       G_CALLBACK (main_window_go_to_previous_page_cb) },
 
@@ -121,12 +149,16 @@ static const GtkActionEntry g_NormalEntries[] =
       N_("Go to the last page"),
       G_CALLBACK (main_window_go_to_last_page_cb) },
 
-    { "About", GTK_STOCK_ABOUT, N_("_About"), NULL, 
-      N_("Display application's credits"), 
-      G_CALLBACK (main_window_about_box_cb) }
+    { "About", GTK_STOCK_ABOUT, N_("_About"), NULL,
+      N_("Display application's credits"),
+      G_CALLBACK (main_window_about_box_cb) },
+
+    // Accelerator keys.
+    { "Slash", GTK_STOCK_FIND, NULL, "slash", NULL,
+      G_CALLBACK (main_window_find_cb) }
 };
 
-static GtkToggleActionEntry g_ToggleEntries[] = 
+static GtkToggleActionEntry g_ToggleEntries[] =
 {
     { "ShowToolBar", NULL, N_("Show _Toolbar"), NULL,
       N_("Show or hide the toolbar"),
@@ -134,16 +166,16 @@ static GtkToggleActionEntry g_ToggleEntries[] =
     { "ShowStatusBar", NULL, N_("Show _Statusbar"), NULL,
       N_("Show or hide the statusbar"),
       G_CALLBACK (main_window_show_statusbar_cb), TRUE },
-    { "ShowIndex", NULL, N_("Show I_ndex"), "<control>I",
+    { "ShowIndex", NULL, N_("Show I_ndex"), "F9",
       N_("Show or hide the document's outline"),
       G_CALLBACK (main_window_show_index_cb), FALSE },
 
     { "ZoomFit", GTK_STOCK_ZOOM_FIT, N_("Zoom to _Fit"), NULL,
-      N_("Make the current document fill the window"), 
+      N_("Make the current document fill the window"),
       G_CALLBACK (main_window_zoom_fit_cb), FALSE },
 
     { "ZoomWidth", EPDFVIEW_STOCK_ZOOM_WIDTH, N_("Zoom to _Width"), NULL,
-      N_("Make the current document fill the window width"), 
+      N_("Make the current document fill the window width"),
       G_CALLBACK (main_window_zoom_width_cb), FALSE }
 
 };
@@ -185,8 +217,14 @@ MainView::MainView (MainPter *pter):
     gtk_toolbar_insert (GTK_TOOLBAR (toolBar), m_CurrentPageToolItem,
                         CURRENT_PAGE_POS);
     gtk_widget_show_all (GTK_WIDGET (m_CurrentPageToolItem));
-    gtk_toolbar_insert (GTK_TOOLBAR (toolBar), gtk_separator_tool_item_new (),
-                        CURRENT_PAGE_POS + 1);
+    GtkToolItem *separator = gtk_separator_tool_item_new ();
+    gtk_toolbar_insert (GTK_TOOLBAR (toolBar), separator, CURRENT_PAGE_POS + 1);
+    gtk_widget_show_all (GTK_WIDGET (separator));
+    // Add the current zoom item.
+    createCurrentZoom ();
+    gtk_toolbar_insert (GTK_TOOLBAR (toolBar), m_CurrentZoomToolItem,
+                        CURRENT_ZOOM_POS);
+    gtk_widget_show_all (GTK_WIDGET (m_CurrentZoomToolItem));
     // Create the page view
     GtkWidget *pageViewPaned = createPageView ();
     gtk_box_pack_start (GTK_BOX (m_MainBox), pageViewPaned, TRUE, TRUE, 0);
@@ -212,7 +250,7 @@ MainView::~MainView ()
 void
 MainView::activeZoomFit (gboolean active)
 {
-    GtkAction *zoomFit = 
+    GtkAction *zoomFit =
         gtk_ui_manager_get_action (m_UIManager, "/MenuBar/ViewMenu/ZoomFit");
     gtk_toggle_action_set_active (GTK_TOGGLE_ACTION (zoomFit), active);
 }
@@ -220,7 +258,7 @@ MainView::activeZoomFit (gboolean active)
 void
 MainView::activeZoomWidth (gboolean active)
 {
-    GtkAction *zoomWidth = 
+    GtkAction *zoomWidth =
         gtk_ui_manager_get_action (m_UIManager, "/MenuBar/ViewMenu/ZoomWidth");
     gtk_toggle_action_set_active (GTK_TOGGLE_ACTION (zoomWidth), active);
 }
@@ -236,13 +274,34 @@ MainView::openFileDialog (const gchar *lastFolder)
             NULL);
     gtk_dialog_set_alternative_button_order (GTK_DIALOG (openDialog),
             GTK_RESPONSE_ACCEPT, GTK_RESPONSE_CANCEL, -1);
-    
+
+    // Select the last used folder as the initial folder, if any.
     if ( NULL != lastFolder )
     {
         gtk_file_chooser_set_current_folder (GTK_FILE_CHOOSER (openDialog),
                                              lastFolder);
     }
-    
+
+    // Add the file type filters.
+    {
+        GtkFileFilter *pdfFilter = gtk_file_filter_new ();
+        gtk_file_filter_set_name (pdfFilter, 
+                                  _("Portable Document Format (PDF) Files"));
+        gtk_file_filter_add_mime_type (pdfFilter, "application/pdf");
+
+        gtk_file_chooser_add_filter (GTK_FILE_CHOOSER (openDialog), pdfFilter);
+        // Set this filter as the default.
+        gtk_file_chooser_set_filter (GTK_FILE_CHOOSER (openDialog), pdfFilter);
+    }
+
+    {
+        GtkFileFilter *anyFilter = gtk_file_filter_new ();
+        gtk_file_filter_set_name (anyFilter, _("All Files"));
+        gtk_file_filter_add_pattern (anyFilter, "*");
+
+        gtk_file_chooser_add_filter (GTK_FILE_CHOOSER (openDialog), anyFilter);
+    }
+
     if ( GTK_RESPONSE_ACCEPT == gtk_dialog_run (GTK_DIALOG (openDialog)) )
     {
         gchar *fileName = gtk_file_chooser_get_filename (
@@ -278,7 +337,7 @@ MainView::promptPasswordDialog ()
     gtk_box_pack_start (GTK_BOX (vbox), secondaryLabel, TRUE, TRUE, 0);
     gtk_box_pack_start (GTK_BOX (vbox), passwordEntry, FALSE, FALSE, 6);
 
-    GtkWidget *image = 
+    GtkWidget *image =
         gtk_image_new_from_stock (GTK_STOCK_DIALOG_AUTHENTICATION,
                                   GTK_ICON_SIZE_DIALOG);
     gtk_misc_set_alignment (GTK_MISC (image), 0.5, 0.0);
@@ -287,7 +346,7 @@ MainView::promptPasswordDialog ()
     gtk_box_pack_start (GTK_BOX (hbox), vbox, TRUE, TRUE, 0);
     gtk_widget_show_all (hbox);
 
-    
+
     GtkWidget *passwordDialog = gtk_dialog_new_with_buttons (
             _("Password"), GTK_WINDOW (m_MainWindow),
             GTK_DIALOG_DESTROY_WITH_PARENT,
@@ -300,7 +359,7 @@ MainView::promptPasswordDialog ()
                                      GTK_RESPONSE_ACCEPT);
     gtk_box_pack_start (GTK_BOX (GTK_DIALOG (passwordDialog)->vbox),
                         hbox, FALSE, FALSE, 0);
-    
+
     gchar *password = NULL;
     if ( GTK_RESPONSE_ACCEPT == gtk_dialog_run (GTK_DIALOG (passwordDialog)) )
     {
@@ -310,7 +369,57 @@ MainView::promptPasswordDialog ()
     return password;
 }
 
-void 
+gchar *
+MainView::saveFileDialog (const gchar *lastFolder)
+{
+    GtkWidget *saveDialog = gtk_file_chooser_dialog_new (_("Save PDF File"),
+            GTK_WINDOW (m_MainWindow),
+            GTK_FILE_CHOOSER_ACTION_SAVE,
+            GTK_STOCK_CANCEL, GTK_RESPONSE_CANCEL,
+            GTK_STOCK_OPEN, GTK_RESPONSE_ACCEPT,
+            NULL);
+    gtk_dialog_set_alternative_button_order (GTK_DIALOG (saveDialog),
+            GTK_RESPONSE_ACCEPT, GTK_RESPONSE_CANCEL, -1);
+
+    // Select the last used folder as the initial folder, if any.
+    if ( NULL != lastFolder )
+    {
+        gtk_file_chooser_set_current_folder (GTK_FILE_CHOOSER (saveDialog),
+                                             lastFolder);
+    }
+
+    // Add the file type filters.
+    {
+        GtkFileFilter *pdfFilter = gtk_file_filter_new ();
+        gtk_file_filter_set_name (pdfFilter, 
+                                  _("Portable Document Format (PDF) Files"));
+        gtk_file_filter_add_mime_type (pdfFilter, "application/pdf");
+
+        gtk_file_chooser_add_filter (GTK_FILE_CHOOSER (saveDialog), pdfFilter);
+        // Set this filter as the default.
+        gtk_file_chooser_set_filter (GTK_FILE_CHOOSER (saveDialog), pdfFilter);
+    }
+
+    {
+        GtkFileFilter *anyFilter = gtk_file_filter_new ();
+        gtk_file_filter_set_name (anyFilter, _("All Files"));
+        gtk_file_filter_add_pattern (anyFilter, "*");
+
+        gtk_file_chooser_add_filter (GTK_FILE_CHOOSER (saveDialog), anyFilter);
+    }
+
+    if ( GTK_RESPONSE_ACCEPT == gtk_dialog_run (GTK_DIALOG (saveDialog)) )
+    {
+        gchar *fileName = gtk_file_chooser_get_filename (
+                                GTK_FILE_CHOOSER (saveDialog));
+        gtk_widget_destroy (saveDialog);
+        return fileName;
+    }
+    gtk_widget_destroy (saveDialog);
+    return NULL;
+}
+
+void
 MainView::sensitiveFind (gboolean sensitive)
 {
     GtkAction *find = 
@@ -318,7 +427,7 @@ MainView::sensitiveFind (gboolean sensitive)
     gtk_action_set_sensitive (find, sensitive);
 }
 
-void 
+void
 MainView::sensitiveGoToFirstPage (gboolean sensitive)
 {
     GtkAction *goToFirstPage = gtk_ui_manager_get_action (m_UIManager,
@@ -326,23 +435,23 @@ MainView::sensitiveGoToFirstPage (gboolean sensitive)
     gtk_action_set_sensitive (goToFirstPage, sensitive);
 }
 
-void 
+void
 MainView::sensitiveGoToLastPage (gboolean sensitive)
 {
-    GtkAction *goToLastPage = 
+    GtkAction *goToLastPage =
         gtk_ui_manager_get_action (m_UIManager, "/MenuBar/GoMenu/GoToLastPage");
     gtk_action_set_sensitive (goToLastPage, sensitive);
 }
 
-void 
+void
 MainView::sensitiveGoToNextPage (gboolean sensitive)
 {
-    GtkAction *goToNextPage = 
+    GtkAction *goToNextPage =
         gtk_ui_manager_get_action (m_UIManager, "/MenuBar/GoMenu/GoToNextPage");
     gtk_action_set_sensitive (goToNextPage, sensitive);
 }
 
-void 
+void
 MainView::sensitiveGoToPage (gboolean sensitive)
 {
     gtk_widget_set_sensitive (GTK_WIDGET (m_CurrentPageToolItem), sensitive);
@@ -356,14 +465,24 @@ MainView::sensitiveGoToPreviousPage (gboolean sensitive)
     gtk_action_set_sensitive (goToPreviousPage, sensitive);
 }
 
-void 
+void
 MainView::sensitiveOpen (gboolean sensitive)
 {
-    GtkAction *open = 
+    GtkAction *open =
         gtk_ui_manager_get_action (m_UIManager, "/MenuBar/FileMenu/OpenFile");
     gtk_action_set_sensitive (open, sensitive);
 }
 
+#if defined (HAVE_CUPS)
+void
+MainView::sensitivePrint (gboolean sensitive)
+{
+    GtkAction *print = gtk_ui_manager_get_action (m_UIManager,
+            "/MenuBar/FileMenu/PrintPlaceHolder/Print");
+    gtk_action_set_sensitive (print, sensitive);
+}
+#endif // HAVE_CUPS
+
 void 
 MainView::sensitiveReload (gboolean sensitive)
 {
@@ -380,7 +499,7 @@ MainView::sensitiveRotateLeft (gboolean sensitive)
     gtk_action_set_sensitive (rotateLeft, sensitive);
 }
 
-void 
+void
 MainView::sensitiveRotateRight (gboolean sensitive)
 {
     GtkAction *rotateRight = gtk_ui_manager_get_action (m_UIManager, 
@@ -388,41 +507,55 @@ MainView::sensitiveRotateRight (gboolean sensitive)
     gtk_action_set_sensitive (rotateRight, sensitive);
 }
 
-void 
+void
+MainView::sensitiveSave (gboolean sensitive)
+{
+    GtkAction *save =
+        gtk_ui_manager_get_action (m_UIManager, "/MenuBar/FileMenu/SaveFile");
+    gtk_action_set_sensitive (save, sensitive);
+}
+
+void
+MainView::sensitiveZoom (gboolean sensitive)
+{
+    gtk_widget_set_sensitive (GTK_WIDGET (m_CurrentZoomToolItem), sensitive);
+}
+
+void
 MainView::sensitiveZoomIn (gboolean sensitive)
 {
-    GtkAction *zoomIn = 
+    GtkAction *zoomIn =
         gtk_ui_manager_get_action (m_UIManager, "/MenuBar/ViewMenu/ZoomIn");
     gtk_action_set_sensitive (zoomIn, sensitive);
 }
 
-void 
+void
 MainView::sensitiveZoomOut (gboolean sensitive)
 {
-    GtkAction *zoomOut = 
+    GtkAction *zoomOut =
         gtk_ui_manager_get_action (m_UIManager, "/MenuBar/ViewMenu/ZoomOut");
     gtk_action_set_sensitive (zoomOut, sensitive);
 }
 
-void 
+void
 MainView::sensitiveZoomFit (gboolean sensitive)
 {
-    GtkAction *zoomFit = 
+    GtkAction *zoomFit =
         gtk_ui_manager_get_action (m_UIManager, "/MenuBar/ViewMenu/ZoomFit");
     gtk_action_set_sensitive (zoomFit, sensitive);
 }
 
-void 
+void
 MainView::sensitiveZoomWidth (gboolean sensitive)
 {
-    GtkAction *zoomWidth = 
+    GtkAction *zoomWidth =
         gtk_ui_manager_get_action (m_UIManager, "/MenuBar/ViewMenu/ZoomWidth");
     gtk_action_set_sensitive (zoomWidth, sensitive);
 }
 
-void 
+void
 MainView::show (void)
-{    
+{
     Config &config = Config::getConfig ();
     gtk_window_set_default_size (GTK_WINDOW (m_MainWindow),
                                  config.getWindowWidth (),
@@ -432,7 +565,7 @@ MainView::show (void)
                      config.getWindowX (), config.getWindowY ());
 }
 
-void 
+void
 MainView::showErrorMessage (const gchar *title, const gchar *body)
 {
     GtkWidget *errorDialog = gtk_message_dialog_new (
@@ -491,13 +624,13 @@ MainView::setCursor (ViewCursor cursorType)
     gdk_flush ();
 }
 
-void 
-MainView::setTotalPages (gint pages)
+void
+MainView::setNumberOfPagesText (const gchar *text)
 {
-    m_TotalPages = pages;
+    gtk_label_set_text (GTK_LABEL (m_NumberOfPages), text);
 }
 
-void 
+void
 MainView::setGoToPageText (const gchar *text)
 {
     gtk_entry_set_text (GTK_ENTRY (m_CurrentPage), text);
@@ -509,6 +642,12 @@ MainView::getGoToPageText (void)
     return gtk_entry_get_text (GTK_ENTRY (m_CurrentPage));
 }
 
+const gchar *
+MainView::getZoomText (void)
+{
+    return gtk_entry_get_text (GTK_ENTRY (m_CurrentZoom));
+}
+
 void
 MainView::setStatusBarText (const gchar *text)
 {
@@ -521,6 +660,12 @@ MainView::setStatusBarText (const gchar *text)
     }
 }
 
+void
+MainView::setZoomText (const gchar *text)
+{
+    gtk_entry_set_text (GTK_ENTRY (m_CurrentZoom), text);
+}
+
 IFindView *
 MainView::getFindView ()
 {
@@ -529,11 +674,25 @@ MainView::getFindView ()
 
 IPageView *
 MainView::getPageView ()
-{    
+{
     return m_PageView;
 }
 
-void 
+IPreferencesView *
+MainView::getPreferencesView ()
+{
+    return new PreferencesView (GTK_WINDOW (m_MainWindow));
+}
+
+#if defined (HAVE_CUPS)
+IPrintView *
+MainView::getPrintView ()
+{
+    return new PrintView (GTK_WINDOW (m_MainWindow));
+}
+#endif // HAVE_CUPS
+
+void
 MainView::setTitle (const gchar *title)
 {
     gtk_window_set_title (GTK_WINDOW (m_MainWindow), title);
@@ -574,7 +733,7 @@ MainView::showToolbar (gboolean show)
                                             "/MenuBar/ViewMenu/ShowToolBar");
     gtk_toggle_action_set_active (GTK_TOGGLE_ACTION (toggleAction), show);
     GtkWidget *toolBar = gtk_ui_manager_get_widget (m_UIManager, "/ToolBar");
-    if ( show ) 
+    if ( show )
     {
         gtk_widget_show (toolBar);
     }
@@ -591,27 +750,50 @@ MainView::showToolbar (gboolean show)
 ///
 /// @brief Creates the "Current Page" widget that will be displayed on toolbar.
 ///
-/// @return The tool item to add to the toolbar.
-///
 void
 MainView::createCurrentPage ()
 {
     GtkWidget *hbox = gtk_hbox_new (FALSE, 3);
-    
-    GtkWidget *pageLabel= gtk_label_new (_("Page"));
+
+    GtkWidget *pageLabel = gtk_label_new (_("Page"));
     gtk_box_pack_start (GTK_BOX (hbox), pageLabel, FALSE, FALSE, 0);
-    
+
     m_CurrentPage = gtk_entry_new ();
-    gtk_entry_set_text (GTK_ENTRY (m_CurrentPage), _("0 of 0"));
+    gtk_entry_set_text (GTK_ENTRY (m_CurrentPage), "0");
     gtk_entry_set_alignment (GTK_ENTRY (m_CurrentPage), 1.0f);
+    gtk_entry_set_width_chars (GTK_ENTRY (m_CurrentPage), CURRENT_PAGE_WIDTH);
     g_signal_connect (G_OBJECT (m_CurrentPage), "activate",
                       G_CALLBACK (main_window_go_to_page_cb), m_Pter);
     gtk_box_pack_start (GTK_BOX (hbox), m_CurrentPage, TRUE, TRUE, 0);
-    
+    m_NumberOfPages = gtk_label_new (_("of 0"));
+    gtk_misc_set_alignment (GTK_MISC (m_NumberOfPages), 0.0f, 0.5f);
+    gtk_box_pack_start (GTK_BOX (hbox), m_NumberOfPages, FALSE, FALSE, 0);
+
     m_CurrentPageToolItem = gtk_tool_item_new ();
     gtk_container_add (GTK_CONTAINER (m_CurrentPageToolItem), hbox);
 }
 
+
+///
+/// @brief Creates the "Current Zoom" widget that will be displayed on toolbar.
+///
+void
+MainView::createCurrentZoom ()
+{
+    m_CurrentZoom = gtk_entry_new ();
+    gtk_entry_set_alignment (GTK_ENTRY (m_CurrentZoom), 1.0f);
+    gtk_entry_set_width_chars (GTK_ENTRY (m_CurrentZoom), CURRENT_ZOOM_WIDTH);
+    g_signal_connect (G_OBJECT (m_CurrentZoom), "activate",
+                      G_CALLBACK (main_window_zoom_cb), m_Pter);
+
+    GtkWidget *zoomBox = gtk_hbox_new (FALSE, 3);
+    gtk_box_pack_start_defaults (GTK_BOX (zoomBox), m_CurrentZoom);
+    gtk_box_pack_start (GTK_BOX (zoomBox), gtk_label_new ("%"),
+                        FALSE, FALSE, 0);
+    m_CurrentZoomToolItem = gtk_tool_item_new ();
+    gtk_container_add (GTK_CONTAINER (m_CurrentZoomToolItem), zoomBox);
+}
+
 ///
 /// @brief Creates the widget that will display the page.
 ///
@@ -627,7 +809,7 @@ MainView::createPageView ()
 {
     // Create the page view.
     m_PageView = new PageView ();
-    
+
     // Create the side bar, with the index tree.
     m_Outline = gtk_tree_store_new (outlineNumColumns, 
                                     G_TYPE_STRING, G_TYPE_INT, G_TYPE_POINTER);
@@ -638,7 +820,7 @@ MainView::createPageView ()
     gtk_tree_view_column_set_expand (column, TRUE);
     gtk_tree_view_column_set_title (column, _("Index"));
     gtk_tree_view_append_column (GTK_TREE_VIEW (m_TreeIndex), column);
-   
+
     {
         GtkCellRenderer *renderer = gtk_cell_renderer_text_new ();
         GValue ellipsizeMode = { 0, };
@@ -649,7 +831,7 @@ MainView::createPageView ()
         gtk_tree_view_column_pack_start (column, renderer, TRUE);
         gtk_tree_view_column_set_attributes (column, renderer, "text", 0, NULL);
     }
-            
+
     {
         GtkCellRenderer *renderer = gtk_cell_renderer_text_new ();
         gtk_tree_view_column_pack_end (column, renderer, FALSE);
@@ -685,7 +867,7 @@ MainView::createPageView ()
 ///
 void
 MainView::createUIManager ()
-{        
+{
     GtkActionGroup *actionGroup = gtk_action_group_new ("ePDFView");
     // Set the translation domain for labels and tool tips.
     gtk_action_group_set_translation_domain (actionGroup, PACKAGE);
@@ -697,10 +879,10 @@ MainView::createUIManager ()
                                          m_Pter);
     m_UIManager = gtk_ui_manager_new ();
     gtk_ui_manager_insert_action_group (m_UIManager, actionGroup, 0);
-    
+
     GError *error = NULL;
-    if ( !gtk_ui_manager_add_ui_from_file (m_UIManager, 
-                                           DATADIR"/ui/epdfview-ui.xml", 
+    if ( !gtk_ui_manager_add_ui_from_file (m_UIManager,
+                                           DATADIR"/ui/epdfview-ui.xml",
                                            &error) )
     {
         g_critical (_("Error building UI manager: %s\n"), error->message);
@@ -708,6 +890,17 @@ MainView::createUIManager ()
         exit (EXIT_FAILURE);
     }
 
+#if defined (HAVE_CUPS)
+    if ( !gtk_ui_manager_add_ui_from_file (m_UIManager,
+                                           DATADIR"/ui/epdfview-ui-print.xml",
+                                           &error) )
+    {
+        g_critical (_("Error building UI manager: %s\n"), error->message);
+        g_error_free (error);
+        exit (EXIT_FAILURE);
+    }
+#endif // HAVE_CUPS
+
     // Set the previous and next pages tool items important, so they will
     // be shown when the toolbar's style is GTK_TOOLBAR_BOTH_HORIZ.
     GtkWidget *prevPage = gtk_ui_manager_get_widget (m_UIManager,
@@ -730,7 +923,7 @@ MainView::setMainWindowIcon ()
         "icon_epdfview-32.png",
         "icon_epdfview-24.png",
     };
-    
+
     GList *iconList = NULL;
     int iconFilesNum = G_N_ELEMENTS (iconFiles);
     for ( int iconIndex = 0 ; iconIndex < iconFilesNum ; iconIndex++ )
@@ -770,11 +963,11 @@ void
 MainView::setOutlineChildren (DocumentOutline *outline, GtkTreeIter *rootIter)
 {
     if ( NULL != outline )
-    {        
+    {
         DocumentOutline *child = outline->getFirstChild ();
         while (NULL != child)
         {
-            GtkTreeIter childIter;            
+            GtkTreeIter childIter;
             gtk_tree_store_append (m_Outline, &childIter, rootIter);
             gtk_tree_store_set (m_Outline, &childIter,
                                 outlineTitleColumn, child->getTitle (),
@@ -980,7 +1173,7 @@ main_window_open_file_cb (GtkWidget *widget, gpointer data)
 ///
 void
 main_window_outline_cb (GtkTreeSelection *treeSelection, gpointer data)
-{    
+{
     g_assert ( NULL != data && "The data parameter is NULL.");
 
     // Since the selection is in single mode, get the currently
@@ -998,6 +1191,33 @@ main_window_outline_cb (GtkTreeSelection *treeSelection, gpointer data)
     }
 }
 
+///
+/// @brief The user wants to change the preferences.
+///
+void
+main_window_preferences_cb (GtkWidget *widget, gpointer data)
+{
+    g_assert ( NULL != data && "The data parameter is NULL.");
+
+    MainPter *pter = (MainPter *)data;
+    pter->preferencesActivated ();
+}
+
+#if defined (HAVE_CUPS)
+///
+///
+/// @brief The user wants to print the current document.
+///
+void
+main_window_print_cb (GtkWidget *widget, gpointer data)
+{
+    g_assert ( NULL != data && "The data parameter is NULL.");
+
+    MainPter *pter = (MainPter *)data;
+    pter->printActivated ();
+}
+#endif // HAVE_CUPS
+
 ///
 /// @brief Called when the window is closed or Quit is activated.
 ///
@@ -1019,6 +1239,18 @@ main_window_show_index_cb (GtkToggleAction *action, gpointer data)
     pter->showIndexActivated (gtk_toggle_action_get_active (action));
 }
 
+///
+/// @brief The user tries to save a copy of the document.
+///
+void
+main_window_save_file_cb (GtkWidget *widget, gpointer data)
+{
+    g_assert ( NULL != data && "The data parameter is NULL.");
+
+    MainPter *pter = (MainPter *)data;
+    pter->saveFileActivated ();
+}
+
 ///
 /// @brief Called when the user clicks on the "Show Statusbar" action.
 ///
@@ -1043,6 +1275,18 @@ main_window_show_toolbar_cb (GtkToggleAction *action, gpointer data)
     pter->showToolbarActivated (gtk_toggle_action_get_active (action));
 }
 
+///
+/// @brief The user tries to set a zoom.
+///
+void
+main_window_zoom_cb (GtkWidget *widget, gpointer data)
+{
+    g_assert ( NULL != data && "The data parameter is NULL.");
+
+    MainPter *pter = (MainPter *)data;
+    pter->zoomActivated ();
+}
+
 ///
 /// @brief The user tries to fit the document into the window.
 ///
